@@ -15,24 +15,24 @@ import (
 	azureutils "github.com/openshift/hive/contrib/pkg/utils/azure"
 )
 
-type azureParams struct {
-	logLevel      string
-	resourceGroup string
+// AzureOptions is the set of options to deprovision an Azure cluster
+type AzureOptions struct {
+	logLevel  string
+	cloudName string
 }
 
 // NewDeprovisionAzureCommand is the entrypoint to create the azure deprovision subcommand
 func NewDeprovisionAzureCommand() *cobra.Command {
-	var params azureParams
-
+	opt := &AzureOptions{}
 	cmd := &cobra.Command{
-		Use:   "azure INFRAID",
+		Use:   "azure INFRAID --azure-cloud-name CLOUDNAME",
 		Short: "Deprovision Azure assets (as created by openshift-installer)",
 		Args:  cobra.ExactArgs(1),
 		Run: func(cmd *cobra.Command, args []string) {
 			if err := validate(); err != nil {
 				log.WithError(err).Fatal("Failed validating Azure credentials")
 			}
-			uninstaller, err := completeAzureUninstaller(args, params)
+			uninstaller, err := completeAzureUninstaller(opt.logLevel, opt.cloudName, args)
 			if err != nil {
 				log.WithError(err).Error("Cannot complete command")
 				return
@@ -42,12 +42,9 @@ func NewDeprovisionAzureCommand() *cobra.Command {
 			}
 		},
 	}
-
 	flags := cmd.Flags()
-	flags.StringVar(&params.logLevel, "loglevel", "info", "log level, one of: debug, info, warn, error, fatal, panic")
-	// default of empty string for the resource group means the uninstall will look for a resource group
-	// matching the infra id.
-	flags.StringVar(&params.resourceGroup, "resource-group", "", "name of Azure resource group (if the installer didn't create one)")
+	flags.StringVar(&opt.logLevel, "loglevel", "info", "log level, one of: debug, info, warn, error, fatal, panic")
+	flags.StringVar(&opt.cloudName, "azure-cloud-name", installertypesazure.PublicCloud.Name(), "cloudName is the name of the Azure cloud environment used to configure the Azure SDK")
 	return cmd
 }
 
@@ -60,10 +57,10 @@ func validate() error {
 	return nil
 }
 
-func completeAzureUninstaller(args []string, params azureParams) (providers.Destroyer, error) {
+func completeAzureUninstaller(logLevel, cloudName string, args []string) (providers.Destroyer, error) {
 
 	// Set log level
-	level, err := log.ParseLevel(params.logLevel)
+	level, err := log.ParseLevel(logLevel)
 	if err != nil {
 		log.WithError(err).Error("cannot parse log level")
 		return nil, err
@@ -82,8 +79,7 @@ func completeAzureUninstaller(args []string, params azureParams) (providers.Dest
 		InfraID: args[0],
 		ClusterPlatformMetadata: types.ClusterPlatformMetadata{
 			Azure: &installertypesazure.Metadata{
-				CloudName:         installertypesazure.PublicCloud,
-				ResourceGroupName: params.resourceGroup,
+				CloudName: installertypesazure.CloudEnvironment(cloudName),
 			},
 		},
 	}
