@@ -29,7 +29,6 @@ import (
 	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/controller"
 	"sigs.k8s.io/controller-runtime/pkg/controller/controllerutil"
-	"sigs.k8s.io/controller-runtime/pkg/event"
 	"sigs.k8s.io/controller-runtime/pkg/handler"
 	"sigs.k8s.io/controller-runtime/pkg/manager"
 	"sigs.k8s.io/controller-runtime/pkg/reconcile"
@@ -45,6 +44,7 @@ import (
 	hiveintv1alpha1 "github.com/openshift/hive/apis/hiveinternal/v1alpha1"
 	"github.com/openshift/hive/pkg/constants"
 	hivemetrics "github.com/openshift/hive/pkg/controller/metrics"
+	"github.com/openshift/hive/pkg/controller/utils"
 	controllerutils "github.com/openshift/hive/pkg/controller/utils"
 	"github.com/openshift/hive/pkg/imageset"
 	"github.com/openshift/hive/pkg/remoteclient"
@@ -179,7 +179,7 @@ func AddToManager(mgr manager.Manager, r reconcile.Reconciler, concurrentReconci
 
 	// Watch for changes to ClusterDeployment
 	err = c.Watch(&source.Kind{Type: &hivev1.ClusterDeployment{}},
-		controllerutils.NewRateLimitedUpdateEventHandler(&handler.EnqueueRequestForObject{}, IsErrorUpdateEvent))
+		controllerutils.NewRateLimitedUpdateEventHandler(&handler.EnqueueRequestForObject{}, controllerutils.IsClusterDeploymentErrorUpdateEvent))
 	if err != nil {
 		logger.WithError(err).Error("Error watching cluster deployment")
 		return err
@@ -494,6 +494,16 @@ func (r *ReconcileClusterDeployment) reconcile(request reconcile.Request, cd *hi
 		return reconcile.Result{}, err
 	}
 
+	if cd.Spec.ManageDNS {
+		changed, err := r.ensureDNSZonePreserveOnDelete(cd, cdLog)
+		if err != nil {
+			return reconcile.Result{}, err
+		}
+		if changed {
+			return reconcile.Result{}, nil
+		}
+	}
+
 	if cd.DeletionTimestamp != nil {
 		if !controllerutils.HasFinalizer(cd, hivev1.FinalizerDeprovision) {
 			// Make sure we have no deprovision underway metric even though this was probably cleared when we
@@ -527,7 +537,7 @@ func (r *ReconcileClusterDeployment) reconcile(request reconcile.Request, cd *hi
 			cdLog.Debugf("cluster expires at: %s", expiry)
 			if time.Now().After(expiry) {
 				cdLog.WithField("expiry", expiry).Info("cluster has expired, issuing delete")
-				err := r.Delete(context.TODO(), cd)
+				err := utils.SafeDelete(r, context.TODO(), cd)
 				if err != nil {
 					cdLog.WithError(err).Log(controllerutils.LogLevel(err), "error deleting expired cluster")
 				}
@@ -557,6 +567,28 @@ func (r *ReconcileClusterDeployment) reconcile(request reconcile.Request, cd *hi
 		return reconcile.Result{}, nil
 	}
 
+	if cd.Spec.ManageDNS {
+		dnsZone, err := r.ensureManagedDNSZone(cd, cdLog)
+		if err != nil {
+			return reconcile.Result{}, err
+		}
+		if dnsZone == nil {
+			// dnsNotReady condition was set.
+			if isSet, _ := isDNSNotReadyConditionSet(cd); isSet {
+				// dnsNotReadyReason is why the dnsNotReady condition was set, therefore requeue so that we check to see if it times out.
+				// add defaultRequeueTime to avoid the race condition where the controller is reconciled at the exact time of the timeout (unlikely, but possible).
+				return reconcile.Result{RequeueAfter: defaultDNSNotReadyTimeout + defaultRequeueTime}, nil
+			}
+
+			return reconcile.Result{}, nil
+		}
+
+		updated, err := r.setDNSDelayMetric(cd, dnsZone, cdLog)
+		if updated || err != nil {
+			return reconcile.Result{}, err
+		}
+	}
+
 	if cd.Spec.Installed {
 		// set installedTimestamp for adopted clusters
 		if cd.Status.InstalledTimestamp == nil {
@@ -592,7 +624,7 @@ func (r *ReconcileClusterDeployment) reconcile(request reconcile.Request, cd *hi
 			if err := r.addOwnershipToSecret(cd, cdLog, cd.Spec.ClusterMetadata.AdminKubeconfigSecretRef.Name); err != nil {
 				return reconcile.Result{}, err
 			}
-			if cd.Spec.ClusterMetadata.AdminPasswordSecretRef.Name != "" {
+			if cd.Spec.ClusterMetadata.AdminPasswordSecretRef != nil && cd.Spec.ClusterMetadata.AdminPasswordSecretRef.Name != "" {
 				if err := r.addOwnershipToSecret(cd, cdLog, cd.Spec.ClusterMetadata.AdminPasswordSecretRef.Name); err != nil {
 					return reconcile.Result{}, err
 				}
@@ -696,28 +728,6 @@ func (r *ReconcileClusterDeployment) reconcile(request reconcile.Request, cd *hi
 		return reconcile.Result{}, nil
 	}
 
-	if cd.Spec.ManageDNS {
-		dnsZone, err := r.ensureManagedDNSZone(cd, cdLog)
-		if err != nil {
-			return reconcile.Result{}, err
-		}
-		if dnsZone == nil {
-			// dnsNotReady condition was set.
-			if isSet, _ := isDNSNotReadyConditionSet(cd); isSet {
-				// dnsNotReadyReason is why the dnsNotReady condition was set, therefore requeue so that we check to see if it times out.
-				// add defaultRequeueTime to avoid the race condition where the controller is reconciled at the exact time of the timeout (unlikely, but possible).
-				return reconcile.Result{RequeueAfter: defaultDNSNotReadyTimeout + defaultRequeueTime}, nil
-			}
-
-			return reconcile.Result{}, nil
-		}
-
-		updated, err := r.setDNSDelayMetric(cd, dnsZone, cdLog)
-		if updated || err != nil {
-			return reconcile.Result{}, err
-		}
-	}
-
 	switch {
 	case cd.Spec.Provisioning != nil:
 		// Ensure the install config matches the ClusterDeployment:
@@ -1426,6 +1436,35 @@ func (r *ReconcileClusterDeployment) setDNSDelayMetric(cd *hivev1.ClusterDeploym
 	return modified, nil
 }
 
+// ensureDNSZonePreserveOnDelete makes sure the DNSZone, if one exists, has a
+// matching PreserveOnDelete setting with its ClusterDeployment. Returns true
+// if a change was made.
+func (r *ReconcileClusterDeployment) ensureDNSZonePreserveOnDelete(cd *hivev1.ClusterDeployment, cdLog log.FieldLogger) (bool, error) {
+	dnsZone := &hivev1.DNSZone{}
+	dnsZoneNamespacedName := types.NamespacedName{Namespace: cd.Namespace, Name: controllerutils.DNSZoneName(cd.Name)}
+	logger := cdLog.WithField("zone", dnsZoneNamespacedName.String())
+	err := r.Get(context.TODO(), dnsZoneNamespacedName, dnsZone)
+	if err != nil && !apierrors.IsNotFound(err) {
+		logger.WithError(err).Error("failed to fetch DNS zone")
+		return false, err
+	} else if err != nil {
+		// DNSZone doesn't exist yet
+		return false, nil
+	}
+
+	if dnsZone.Spec.PreserveOnDelete != cd.Spec.PreserveOnDelete {
+		logger.WithField("preserveOnDelete", cd.Spec.PreserveOnDelete).Info("setting DNSZone PreserveOnDelete to match ClusterDeployment PreserveOnDelete")
+		dnsZone.Spec.PreserveOnDelete = cd.Spec.PreserveOnDelete
+		err := r.Update(context.TODO(), dnsZone)
+		if err != nil {
+			logger.WithError(err).Log(controllerutils.LogLevel(err), "error updating DNSZone")
+			return false, err
+		}
+		return true, nil
+	}
+	return false, nil
+}
+
 func (r *ReconcileClusterDeployment) ensureManagedDNSZone(cd *hivev1.ClusterDeployment, cdLog log.FieldLogger) (*hivev1.DNSZone, error) {
 	switch p := cd.Spec.Platform; {
 	case p.AWS != nil:
@@ -1523,6 +1562,7 @@ func (r *ReconcileClusterDeployment) createManagedDNSZone(cd *hivev1.ClusterDepl
 		},
 		Spec: hivev1.DNSZoneSpec{
 			Zone:               cd.Spec.BaseDomain,
+			PreserveOnDelete:   cd.Spec.PreserveOnDelete,
 			LinkToParentDomain: true,
 		},
 	}
@@ -1551,6 +1591,7 @@ func (r *ReconcileClusterDeployment) createManagedDNSZone(cd *hivev1.ClusterDepl
 		dnsZone.Spec.Azure = &hivev1.AzureDNSZoneSpec{
 			CredentialsSecretRef: cd.Spec.Platform.Azure.CredentialsSecretRef,
 			ResourceGroupName:    cd.Spec.Platform.Azure.BaseDomainResourceGroupName,
+			CloudName:            cd.Spec.Platform.Azure.CloudName,
 		}
 	}
 
@@ -1668,6 +1709,7 @@ func generateDeprovision(cd *hivev1.ClusterDeployment) (*hivev1.ClusterDeprovisi
 	case cd.Spec.Platform.Azure != nil:
 		req.Spec.Platform.Azure = &hivev1.AzureClusterDeprovision{
 			CredentialsSecretRef: &cd.Spec.Platform.Azure.CredentialsSecretRef,
+			CloudName:            &cd.Spec.Platform.Azure.CloudName,
 		}
 	case cd.Spec.Platform.GCP != nil:
 		req.Spec.Platform.GCP = &hivev1.GCPClusterDeprovision{
@@ -2126,43 +2168,3 @@ func LoadReleaseImageVerifier(config *rest.Config) (verify.Interface, error) {
 	}
 	return verify.NewFromManifests([]manifest.Manifest{m}, sigstore.NewCachedHTTPClientConstructor(sigstore.DefaultClient, nil).HTTPClient)
 }
-
-// IsErrorUpdateEvent returns true when the update event is from
-// error state in clusterdeployment.
-func IsErrorUpdateEvent(evt event.UpdateEvent) bool {
-	new, ok := evt.ObjectNew.(*hivev1.ClusterDeployment)
-	if !ok {
-		return false
-	}
-	if len(new.Status.Conditions) == 0 {
-		return false
-	}
-
-	old, ok := evt.ObjectOld.(*hivev1.ClusterDeployment)
-	if !ok {
-		return false
-	}
-
-	for _, cond := range new.Status.Conditions {
-		if controllerutils.IsConditionInDesiredState(cond) {
-			continue
-		}
-
-		oldcond := controllerutils.FindClusterDeploymentCondition(old.Status.Conditions, cond.Type)
-		if oldcond == nil {
-			return true // newly added condition in failed state
-		}
-
-		if controllerutils.IsConditionInDesiredState(*oldcond) {
-			return true // newly Failed condition
-		}
-
-		if cond.Message != oldcond.Message ||
-			cond.Reason != oldcond.Reason {
-			return true // already failing but change in error reported
-		}
-
-	}
-
-	return false
-}
